/**
 * This software is copyright (c) 2019 by
 * 
 * TextGrid Consortium (http://www.textgrid.de)
 * 
 * DAASI International GmbH (http://www.daasi.de)
 *
 * This is free software. You can redistribute it and/or modify it under the terms described in the
 * GNU Lesser General Public License v3 of which you should have received a copy. Otherwise you can
 * download it from
 *
 * http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * @copyright TextGrid Consortium (http://www.textgrid.de)
 * @copyright DAASI International GmbH (http://www.daasi.de)
 * @license GNU Lesser General Public License v3 (http://www.gnu.org/licenses/lgpl-3.0.txt)
 * @author Stefan E. Funk (stefan.e.funk@daasi.de)
 */

package info.textgrid.middleware.tgauth.tgauthclient;

import static org.junit.Assert.assertTrue;
import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.util.List;
import java.util.Properties;
import javax.xml.ws.soap.SOAPFaultException;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.namespaces.middleware.tgauth.AuthenticationFault;
import info.textgrid.namespaces.middleware.tgauth.BooleanResponse;
import info.textgrid.namespaces.middleware.tgauth.CreateProjectRequest;
import info.textgrid.namespaces.middleware.tgauth.CreateProjectResponse;
import info.textgrid.namespaces.middleware.tgauth.DeleteProjectRequest;
import info.textgrid.namespaces.middleware.tgauth.GetAllProjectsRequest;
import info.textgrid.namespaces.middleware.tgauth.GetIDsRequest;
import info.textgrid.namespaces.middleware.tgauth.GetIDsResponse;
import info.textgrid.namespaces.middleware.tgauth.GetMyUserAttributesRequest;
import info.textgrid.namespaces.middleware.tgauth.GetMyUserAttributesResponse;
import info.textgrid.namespaces.middleware.tgauth.GetNamesRequest;
import info.textgrid.namespaces.middleware.tgauth.GetNamesResponse;
import info.textgrid.namespaces.middleware.tgauth.GetObjectsRequest;
import info.textgrid.namespaces.middleware.tgauth.GetProjectDescriptionRequest;
import info.textgrid.namespaces.middleware.tgauth.GetProjectDescriptionResponse;
import info.textgrid.namespaces.middleware.tgauth.GetRightsRequest;
import info.textgrid.namespaces.middleware.tgauth.GetSupportedUserAttributesRequest;
import info.textgrid.namespaces.middleware.tgauth.GetSupportedUserAttributesResponse;
import info.textgrid.namespaces.middleware.tgauth.GetUserRoleRequest;
import info.textgrid.namespaces.middleware.tgauth.GetUserRoleResponse;
import info.textgrid.namespaces.middleware.tgauth.NotEmptyFault;
import info.textgrid.namespaces.middleware.tgauth.OperationsetResponse;
import info.textgrid.namespaces.middleware.tgauth.PortTgextra;
import info.textgrid.namespaces.middleware.tgauth.ProjectInfo;
import info.textgrid.namespaces.middleware.tgauth.RbacFault;
import info.textgrid.namespaces.middleware.tgauth.ResourcesetResponse;
import info.textgrid.namespaces.middleware.tgauth.RolesetResponse;
import info.textgrid.namespaces.middleware.tgauth.TgAssignedRolesRequest;
import info.textgrid.namespaces.middleware.tgauth.TgCheckAccessRequest;
import info.textgrid.namespaces.middleware.tgauth.UnknownProjectFault;
import info.textgrid.namespaces.middleware.tgauth.UserAttribute;
import info.textgrid.namespaces.middleware.tgauth.UserDetail;
import info.textgrid.namespaces.middleware.tgauth.UserExistsRequest;
import info.textgrid.namespaces.middleware.tgauth.UserRole;
import info.textgrid.namespaces.middleware.tgauth_crud.GetEPPNRequest;
import info.textgrid.namespaces.middleware.tgauth_crud.GetEPPNResponse;
import info.textgrid.namespaces.middleware.tgauth_crud.PortTgextraCrud;
import info.textgrid.namespaces.middleware.tgauth_crud.TgCrudCheckAccessRequest;
import info.textgrid.namespaces.middleware.tgauth_crud.TgCrudCheckAccessResponse;
import info.textgrid.namespaces.middleware.tgauth_crud.UnknownResourceFault;

/**
 * TODOLOG
 * 
 * TODO Implement empty method stubs!
 * 
 **
 * CHANGELOG
 * 
 * 2019-01-28 - Funk - Prettyfying output.
 * 
 * 2018-11-23 - Funk - Adding tests for issue #26471
 * 
 * 2016-08-11 - Funk - Using TGAuth client from TGAuth service utilities now!
 * 
 */

/**
 * <p>
 * This are TG-auth ONLINE tests for testing a running tgauth service to fulfill its duties.
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2019-01-28
 * @since 2010-09-07
 */

// @Ignore
public class TestTGAuthServiceOnline {

  // **
  // STATIC FINALS
  // **

  private static final String PROPERTIES_FILE = "tgauth.test.textgridlab-org.properties";
  // private static final String PROPERTIES_FILE = "tgauth.test.dev-textgridlab-org.properties";
  private static final String NOT_YET_IMPLEMENTED = "test not yet implemented!";
  private static final int TGEXTRA_CLIENT_TIMEOUT = 0;
  private static final String ERROR = "\t??ERROR: ";
  private static final String OK = "\tOK";
  private static final String OPERATION_CREATE = "create";

  // **
  // STATICS
  // **

  private static PortTgextra tgextra;
  private static PortTgextraCrud tgextraCrud;
  private static String rbacSessionId;
  private static String rbacSessionIdInvalid;
  private static String crudSecret;
  private static String crudSecretInvalid;
  private static String logParameter;
  private static String projectId;
  private static String projectIdInvalid;
  private static String username;
  private static String usernameInvalid;
  private static String usernameSearch;
  private static String junitProjectName;
  private static String getMyUserAtrributesEmail;

  // **
  // PREPARATIONS
  // **

  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception {

    // Load properties file.
    Properties p = new Properties();
    p.load(new FileInputStream(TGAuthClientCommon.getResource(PROPERTIES_FILE)));

    // Get and create the tgextra stubs.
    URL location = new URL(p.getProperty("tgextraServiceEndpointUrl"));
    URL locationCrud = new URL(p.getProperty("tgextraCrudServiceEndpointUrl"));

    System.out
        .println("WSDL location tgextra: " + location.getProtocol() + "://" + location.getHost()
            + ":" + (location.getPort() == -1 ? "80" : location.getPort()) + location.getPath());
    System.out.println("WSDL location tgextra-crud: " + locationCrud.getProtocol() + "://"
        + locationCrud.getHost() + ":"
        + (locationCrud.getPort() == -1 ? "80" : locationCrud.getPort()) + locationCrud.getPath());

    // initTgextraService(location, 0);
    initTgextraService(location, TGEXTRA_CLIENT_TIMEOUT);
    // initTgextraCrudService(locationCrud, 0);
    initTgextraCrudService(locationCrud, TGEXTRA_CLIENT_TIMEOUT);

    // Create project name.
    File tempFile = File.createTempFile("JUNIT_", "_01");
    junitProjectName = tempFile.getName();
    boolean tempFileDeleted = tempFile.delete();
    if (!tempFileDeleted) {
      System.err
          .println(ERROR + "temp file '" + tempFile.getCanonicalPath() + "' could not be deleted");
    }
    // Get other needed things.
    rbacSessionId = p.getProperty("rbacSessionId");
    rbacSessionIdInvalid = p.getProperty("rbacSessionIdInvalid");
    logParameter = p.getProperty("logParameter");
    projectIdInvalid = p.getProperty("projectIdInvalid");
    crudSecret = p.getProperty("crudSecret");
    crudSecretInvalid = p.getProperty("crudSecretInvalid");
    username = p.getProperty("username");
    usernameInvalid = p.getProperty("usernameInvalid");
    usernameSearch = p.getProperty("usernameSearch");
    getMyUserAtrributesEmail = p.getProperty("getMyUserAtrributesEmail");
    addProject();
  }

  /**
   * @throws Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception {
    // deleteObjects();
    deleteProject();
  }

  // **
  // TEST BUGS
  // **
  @Test
  public void testBug21896() {

    System.out.println("TESTING #BUG21896");

    //
    // Retrieve project information.
    //
    System.out.println("\tgetting project info for ID " + projectId);

    GetProjectDescriptionRequest req = new GetProjectDescriptionRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setProject(projectId);

    GetProjectDescriptionResponse res = tgextra.getProjectDescription(req);

    System.out.println("\tproject name:        " + res.getProject().getName());
    System.out.println("\tproject ID:          " + res.getProject().getId());
    System.out.println("\tproject Description: " + res.getProject().getDescription());
    System.out.println("\tproject File:        " + res.getProject().getFile());
  }

  // **
  // TGEXTRA TESTS
  // **

  /**
   * <p>
   * Test the userExists method with valid user.
   * </p>
   * 
   * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
   */
  @Test
  public void testTgextraUserExists()
      throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault {

    System.out.println("TESTING #USEREXISTS with existing username");

    UserExistsRequest req = new UserExistsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setUsername(username);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tusername:      " + req.getUsername());
    System.out.println(TGAuthClientCommon.TABLINE);

    boolean result = tgextra.userExists(req).isResult();
    System.out.println("\tresult: " + result);

    if (!result) {
      assertTrue(false);
    }

    System.out.println(TGAuthClientCommon.TABLINE);
    System.out.println(OK);
  }

  /**
   * <p>
   * Test the userExists method with valid user.
   * </p>
   * 
   * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
   */
  @Test
  public void testTgextraUserExistsInvalidUsername()
      throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault {

    System.out.println("TESTING #USEREXISTS with invalid username");

    UserExistsRequest req = new UserExistsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setUsername(usernameInvalid);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tusername:      " + req.getUsername());
    System.out.println(TGAuthClientCommon.TABLINE);

    boolean result = tgextra.userExists(req).isResult();
    System.out.println("\tresult: " + result);

    if (result) {
      assertTrue(false);
    }

    System.out.println(TGAuthClientCommon.TABLINE);
    System.out.println(OK);
  }

  /**
   * 
   */
  @Test
  public void testGetNames() {

    System.out.println("TESTING #GETNAMES");

    // Create getNames request.
    GetNamesRequest req = new GetNamesRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.getEPPN().add(username);
    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tuser name:     " + req.getEPPN());
    System.out.println(TGAuthClientCommon.TABLINE);

    GetNamesResponse res = tgextra.getNames(req);

    if (res.getUserdetails().size() != 0) {
      assertTrue(false);
    }

    System.out.println("\tuser details list size = " + res.getUserdetails().size());
    System.out.println(OK);
  }

  /**
   * <p>
   * Test searching for names and email addresses.
   * </p>
   * 
   * @throws AuthenticationFault
   */
  @Test
  public void testTgextraGetIDs() {

    System.out.println("TESTING #GETID name");

    // Create getEPPN request.
    GetIDsRequest req = new GetIDsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setName(username);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tusername:      " + req.getName());
    System.out.println(TGAuthClientCommon.TABLINE);

    GetIDsResponse res = tgextra.getIDs(req);

    // Get first (and only) entry for given ePPn.
    UserDetail det = res.getUserdetails().get(0);
    System.out.println(
        "\tuser details:  " + det.getEPPN() + " " + det.getMail() + " " + det.getName() + " "
            + det.getOrganisation() + " " + det.isAgreesearch() + " " + det.isUsersupplieddata());

    System.out.println(TGAuthClientCommon.TABLINE);

    if (!det.getEPPN().equals(username)) {
      System.out.println(ERROR);
      assertTrue(false);
    } else {
      System.out.println(OK);
    }
  }

  /**
   * <p>
   * Test searching for agreeSearch attribute.
   * </p>
   * 
   * @throws AuthenticationFault
   */
  @Test
  public void testTgextraGetIDsAgreeSearch() {

    System.out.println("TESTING #GETID agreeSearch");

    // Create getEPPN request.
    GetIDsRequest req = new GetIDsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setName(username);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tusername:      " + req.getName());
    System.out.println(TGAuthClientCommon.TABLINE);

    GetIDsResponse res = tgextra.getIDs(req);

    // Get first (and only) entry for given ePPn.
    UserDetail det = res.getUserdetails().get(0);
    System.out.println(
        "\tuser details:  " + det.getEPPN() + " " + det.getMail() + " " + det.getName() + " "
            + det.getOrganisation() + " " + det.isAgreesearch() + " " + det.isUsersupplieddata());

    System.out.println(TGAuthClientCommon.TABLINE);

    if (det.isAgreesearch() == null) {
      System.err.println(ERROR + "attribtue agreeSearch is not set");
      assertTrue(false);
    } else {
      System.out.println(OK);
    }
  }

  /**
   * <p>
   * Test searching for part of username, truncated!
   * </p>
   * 
   * @throws AuthenticationFault
   */
  @Test
  public void testTgextraGetIDsSearchUserTruncated() {

    System.out.println("TESTING #GETID search user truncated");

    // Create getEPPN request.
    GetIDsRequest req = new GetIDsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    String trunk = "*" + usernameSearch + "*";
    req.setName(trunk);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tusername:      " + req.getName());
    System.out.println(TGAuthClientCommon.TABLINE);

    GetIDsResponse res = tgextra.getIDs(req);

    int userListSize = res.getUserdetails().size();

    // Get first (and only) entry for given ePPn.
    if (userListSize != 0) {
      System.out.println("\tuser details:");
      for (UserDetail det : res.getUserdetails()) {
        System.out.println("\t\t" + det.getEPPN() + " " + det.getMail() + " " + det.getName() + " "
            + det.getOrganisation() + " " + det.isAgreesearch() + " " + det.isUsersupplieddata());
      }
    } else {
      System.out.println("\tnothing found");
    }

    System.out.println(TGAuthClientCommon.TABLINE);

    if (userListSize == 0) {
      System.err.println(ERROR + "no users found containing " + trunk + "!");
      assertTrue(false);
    } else if (userListSize != 4) {
      System.err.println(ERROR + "expecting 4 users" + trunk + "! Got only " + userListSize);
      assertTrue(false);
    } else {
      System.out.println(OK);
    }
  }

  /**
   * <p>
   * Test searching for part of username, not truncated!
   * </p>
   * 
   * @throws AuthenticationFault
   */
  @Test
  public void testTgextraGetIDsSearchUserNoTruncated() {

    System.out.println("TESTING #GETID search user not truncated");

    // Create getEPPN request.
    GetIDsRequest req = new GetIDsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setName(usernameSearch);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tusername:      " + req.getName());
    System.out.println(TGAuthClientCommon.TABLINE);

    GetIDsResponse res = tgextra.getIDs(req);

    int userListSize = res.getUserdetails().size();

    // Get first (and only) entry for given ePPn.
    if (userListSize != 0) {
      System.out.println("\tuser details:");
      for (UserDetail det : res.getUserdetails()) {
        System.out.println("\t\t" + det.getEPPN() + " " + det.getMail() + " " + det.getName() + " "
            + det.getOrganisation() + " " + det.isAgreesearch() + " " + det.isUsersupplieddata());
      }
    } else {
      System.out.println("nothing found");
    }

    System.out.println(TGAuthClientCommon.TABLINE);

    if (userListSize == 0) {
      System.err.println(ERROR + "no users found containing " + usernameSearch + "!");
      assertTrue(false);
    } else if (userListSize != 3) {
      System.err
          .println(ERROR + "expecting 3 users" + usernameSearch + "! Got only " + userListSize);
      assertTrue(false);
    } else {
      System.out.println(OK);
    }
  }

  /**
   * <p>
   * Test getting all the objects from one project.
   * </p>
   * 
   */
  @Test
  public void testTgextraGetObjects() {

    System.out.println("TESTING #GETOBJECTS objects from project: " + projectId);

    GetObjectsRequest req = new GetObjectsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setProject(projectId);

    try {
      ResourcesetResponse res = tgextra.getObjects(req);
      int size = res.getResource().size();

      System.out.println("\tproject has got " + size + " entr" + (size != 1 ? "ies" : "y"));
      System.out.println(OK);

    } catch (Exception e) {
      printError(e);
      assertTrue(false);
    }
  }

  /**
   * <p>
   * Test the tgCheckAccess method with access granted on project#CREATE.
   * </p>
   * 
   * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
   */
  @Test
  public void testTgextraTgCheckAccessProjectCreate()
      throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault {

    System.out.println("TESTING #TGCHECKACCESS with access granted (project#CREATE)");

    TgCheckAccessRequest req = new TgCheckAccessRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
    req.setResource(projectId);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\toperation:     " + req.getOperation());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      System.out.println("\tresult: " + tgextra.tgCheckAccess(req).isResult());

      System.out.println(TGAuthClientCommon.TABLINE);
      System.out.println(OK);

    } catch (info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault e) {
      printError(e);
      throw e;
    }
  }

  /**
   * <p>
   * Test the tgCheckAccess method with invalid SID.
   * </p>
   * 
   * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
   */
  @Test
  public void testTgextraTgCheckAccessDenied()
      throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault {

    System.out.println("TESTING #TGCHECKACCESS with access denied (project#CREATE)");

    TgCheckAccessRequest req = new TgCheckAccessRequest();
    req.setAuth(rbacSessionIdInvalid);
    req.setLog(logParameter);
    req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
    req.setResource(projectId);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\toperation:     " + req.getOperation());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      boolean result = tgextra.tgCheckAccess(req).isResult();
      System.out.println("\tresult: " + result);

      if (result) {
        assertTrue(false);
      }

      System.out.println(TGAuthClientCommon.TABLINE);
      System.out.println(OK);

    } catch (info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault e) {
      printError(e);
      throw e;
    }
  }

  /**
   * <p>
   * Test the tgCheckAccess method with invalid resource.
   * </p>
   * 
   * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
   */
  @Test(expected = info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault.class)
  public void testTgextraTgCheckAccessUnknownResource()
      throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault {

    System.out.println("TESTING #TGCHECKACCESS with unknown resource (project#CREATE)");

    TgCheckAccessRequest req = new TgCheckAccessRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
    req.setResource(projectIdInvalid);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\toperation:     " + req.getOperation());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      tgextra.tgCheckAccess(req).isResult();
    } catch (info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault e) {
      System.out.println(OK);
      throw e;
    }
  }

  /**
   * <p>
   * Tests if a SID's user has assigned some roles.
   * </p>
   */
  @Test
  public void testTgextraTgAssignedRoles() {

    System.out.println("TESTING #TGASSIGNEDROLES");

    TgAssignedRolesRequest req = new TgAssignedRolesRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println(TGAuthClientCommon.TABLINE);

    RolesetResponse response = tgextra.tgAssignedRoles(req);

    for (String role : response.getRole()) {
      String data[] = role.split(",");
      if (data.length == 3) {
        System.out.println("\t" + data[0] + "\t" + data[1] + "\t" + data[2]);
      }
    }

    System.out.println(TGAuthClientCommon.TABLINE);
    System.out.println(OK);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraTgAssignedProjects() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetAllProjects() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetDeactivatedProjects() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetLeader() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraFilterBySid() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetOwner() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testGetMembers() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * <p>
   * Gets all roles a user has on the project.
   * </p>
   * 
   * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
   * @throws UnknownProjectFault
   */
  @Test
  public void testTgextraGetUserRole()
      throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault, UnknownProjectFault {

    System.out.println("TESTING #GETUSERROLE");

    GetUserRoleRequest req = new GetUserRoleRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setProject(projectId);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tproject:       " + req.getProject());
    System.out.println(TGAuthClientCommon.TABLINE);

    GetUserRoleResponse response = tgextra.getUserRole(req);

    for (UserRole role : response.getUserRole()) {
      System.out.print("\troles for " + role.getUsername() + ": ");
      for (String r : role.getRoles()) {
        System.out.print(r + " ");
      }
      System.out.println();
    }

    System.out.println(TGAuthClientCommon.TABLINE);
    System.out.println(OK);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetFriends() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * <p>
   * Gets all rights a user has on the project.
   * </p>
   * 
   * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
   */
  @Test
  public void testTgextraGetRights()
      throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault {

    System.out.println("TESTING #GETRIGHTS");

    GetRightsRequest req = new GetRightsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setResource(projectId);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println(TGAuthClientCommon.TABLINE);

    OperationsetResponse response = tgextra.getRights(req);

    for (String operation : response.getOperation()) {
      System.out.println("\t" + operation);
    }

    System.out.println(TGAuthClientCommon.TABLINE);
    System.out.println(OK);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraIsPublic() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetNumberOfResources() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetProjectDescription() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetSid() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  public void testTgextraGetSupportedUserAttributes() {

    System.out.println("TESTING #GETSUPPORTEDUSERATTRIBUTES");

    GetSupportedUserAttributesRequest req = new GetSupportedUserAttributesRequest();

    System.out.println(TGAuthClientCommon.TABLINE);

    GetSupportedUserAttributesResponse res = tgextra.getSupportedUserAttributes(req);

    List<UserAttribute> attributeList = res.getAttribute();

    for (UserAttribute a : attributeList) {
      System.out.println(
          "\tattribute: " + a.getDisplayname() + " (" + a.getName() + " / " + a.getValue() + ")");
      System.out.println("\t\tdescription: " + a.getDescription());
    }

    if (attributeList.size() != 16) {
      System.err.println(ERROR + attributeList.size() + " != 16");
      assertTrue(false);
    }

    System.out.println(TGAuthClientCommon.TABLINE);
    System.out.println(OK);
  }

  /**
   * @throws AuthenticationFault
   */
  @Test
  public void testTgextraGetMyUserAttributes() throws AuthenticationFault {

    System.out.println("TESTING #GETMYUSERATTRIBUTES");

    GetMyUserAttributesRequest req = new GetMyUserAttributesRequest();
    req.setAuth(rbacSessionId);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println(TGAuthClientCommon.TABLINE);

    GetMyUserAttributesResponse response = tgextra.getMyUserAttributes(req);

    List<UserAttribute> attributeList = response.getAttribute();

    String email = "";
    for (UserAttribute a : attributeList) {
      System.out.println(
          "\tattribute: " + a.getDisplayname() + " (" + a.getName() + " / " + a.getValue() + ")");
      System.out.println("\t\tdescription: " + a.getDescription());
      if (a.getName().equals("mail")) {
        email = a.getValue();
      }
    }

    if (attributeList.size() != 16 || !email.equals(getMyUserAtrributesEmail)) {
      System.err.println(ERROR + email + ": " + attributeList.size());
      assertTrue(false);
    }

    System.out.println(TGAuthClientCommon.TABLINE);
    System.out.println(OK);
  }

  /**
   * <p>
   * Test the permission the delete a project.
   * </p>
   *
   * @throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault
   * @throws UnknownProjectFault
   * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
   */
  @Test
  public void testTgextraTgDeleteProjectPermission()
      throws info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault,
      info.textgrid.namespaces.middleware.tgauth.AuthenticationFault, UnknownProjectFault {

    System.out.println("TESTING delete project permission");

    // Get roles first for project.
    GetUserRoleRequest req = new GetUserRoleRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setProject(projectId);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tproject:       " + req.getProject());
    System.out.println(TGAuthClientCommon.TABLINE);

    GetUserRoleResponse response = tgextra.getUserRole(req);

    // Check if the user has the role Project Manager in the role project.
    boolean userHasRoleProjektleiter = false;
    for (UserRole role : response.getUserRole()) {
      if (role.getUsername().equals(username)) {
        for (String r : role.getRoles()) {
          if (r.equals(TGAuthClientCommon.ROLE_MANAGER)) {
            userHasRoleProjektleiter = true;
            System.out.println("\trole '" + TGAuthClientCommon.ROLE_MANAGER
                + "' found! deletion of project allowed!");
          }
        }
      }
    }

    if (!userHasRoleProjektleiter) {
      System.err.println(ERROR + "user is not allowed to delete project");
      assertTrue(false);
    }

    System.out.println(TGAuthClientCommon.TABLINE);
    System.out.println(OK);
  }

  // **
  // TGEXTRA CRUD TESTS
  // **

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraCRUDGetCSR() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * <p>
   * Test getting the ePPn. No secret needed anymore!
   * </p>
   * 
   * @throws AuthenticationFault
   */
  @Test
  public void testTgextraCRUDGetEppn()
      throws info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault {

    System.out.println("TESTING #TGCRUDGETEPPN");

    // Create getEPPN request.
    GetEPPNRequest req = new GetEPPNRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\tsecret:        " + req.getSecret());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      GetEPPNResponse response = tgextraCrud.getEPPN(req);

      System.out.println("\teppn: " + response.getEppn());

      System.out.println(TGAuthClientCommon.TABLINE);
      System.out.println(OK);

    } catch (info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault e) {
      printError(e);
      throw e;
    }
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraCRUDGetUUID() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraCRUDNearlyPublish() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraCRUDPublish() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraCRUDPutCRT() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraCRUDRegisterResource() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  /**
   * <p>
   * Test the tgCrudCheckAccess with invalid resource.
   * </p>
   * 
   * @throws UnknownResourceFault
   * @throws info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault
   */
  @Test(expected = UnknownResourceFault.class)
  public void testTgextraCRUDTgCrudCheckAccessUnknownResource() throws UnknownResourceFault,
      info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault {

    System.out.println("TESTING #TGCRUDCHECKACCESS with unknown project resource (project#CREATE)");

    // Create checkAccess request.
    TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
    req.setResource(projectIdInvalid);
    req.setSecret(crudSecret);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\toperation:     " + req.getOperation());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println("\tsecret:        " + req.getSecret());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      tgextraCrud.tgCrudCheckAccess(req);

    } catch (SOAPFaultException e) {
      if (e.getMessage().equals("The given resource is unknown to the system.")) {

        System.out.println(OK);

        throw new UnknownResourceFault(e.getMessage());
      }
    } catch (info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault e) {
      printError(e);
      throw e;
    }
  }

  /**
   * <p>
   * Test the tgCrudCheckAccess with invalid TG-crud secret.
   * </p>
   * 
   * @throws AuthenticationFault
   * @throws UnknownResourceFault
   */
  @Test(expected = info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault.class)
  public void testTgextraCRUDTgCrudCheckAccessNoSecret()
      throws info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault,
      UnknownResourceFault {

    System.out.println("TESTING #TGCRUDCHECKACCESS with invalid tgcrud secret (project#CREATE)");

    // Create checkAccess request.
    TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
    req.setResource(projectId);
    req.setSecret(crudSecretInvalid);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\toperation:     " + req.getOperation());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println("\tsecret:        " + req.getSecret());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      tgextraCrud.tgCrudCheckAccess(req);

    } catch (SOAPFaultException e) {
      if (e.getMessage().equals("You are not cruddy enough - Go away and come back as CRUD!")) {

        System.out.println(OK);

        throw new info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault(
            e.getMessage());
      }
    } catch (UnknownResourceFault e) {
      printError(e);
      throw e;
    }
  }

  /**
   * <p>
   * Test the tgCheckAccess method with access granted on project#CREATE.
   * </p>
   * 
   * @throws UnknownResourceFault
   * @throws info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault
   */
  @Test
  public void testTgextraCRUDTgCrudCheckAccessProjectCreate() throws UnknownResourceFault,
      info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault {

    System.out.println("TESTING #TGCRUDCHECKACCESS with access granted (project#CREATE)");

    TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
    req.setAuth(rbacSessionId);
    req.setSecret(crudSecret);
    req.setLog(logParameter);
    req.setOperation(TGAuthClientCommon.OPERATION_CREATE);
    req.setResource(projectId);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tcrud secret:   " + req.getSecret());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\toperation:     " + req.getOperation());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      TgCrudCheckAccessResponse response = tgextraCrud.tgCrudCheckAccess(req);
      System.out.println("\tresult: " + response.isResult());

      System.out.println(TGAuthClientCommon.TABLINE);
      System.out.println(OK);

    } catch (UnknownResourceFault
        | info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault e) {
      printError(e);
      throw e;
    }
  }

  /**
   * <p>
   * Test the tgCrudCheckAccess.
   * </p>
   * 
   * @throws UnknownResourceFault
   * @throws info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault
   */
  @Test
  public void testTgextraCRUDTgCrudCheckAccessGranted() throws UnknownResourceFault,
      info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault {

    System.out.println("TESTING #TGCRUDCHECKACCESS with access granted");

    // Create checkAccess request.
    TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setOperation(OPERATION_CREATE);
    req.setResource(projectId);
    req.setSecret(crudSecret);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\toperation:     " + req.getOperation());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println("\tsecret:        " + req.getSecret());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      TgCrudCheckAccessResponse response = tgextraCrud.tgCrudCheckAccess(req);

      System.out.println("\tusername:  " + response.getUsername());
      System.out.println("\toperation: " + response.getOperation());
      System.out.println("\tproject:   " + response.getProject().getId() + " \""
          + response.getProject().getName() + "\"");
      System.out.println("\tresult:    " + response.isResult());
      System.out.println("\tisPublic:  " + response.isPublic());

      System.out.println(TGAuthClientCommon.TABLINE);
      System.out.println(OK);

    } catch (UnknownResourceFault
        | info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault e) {
      printError(e);
      throw e;
    }
  }

  /**
   * <p>
   * Test the tgCrudCheckAccess with invalid SID.
   * </p>
   * 
   * @throws UnknownResourceFault
   * @throws info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault
   */
  @Test
  public void testTgextraCRUDTgCrudCheckAccessDenied() throws UnknownResourceFault,
      info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault {

    System.out.println("TESTING #TGCRUDCHECKACCESS with invalid session ID");

    // Create checkAccess request.
    TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
    req.setAuth(rbacSessionIdInvalid);
    req.setLog(logParameter);
    req.setOperation(OPERATION_CREATE);
    req.setResource(projectId);
    req.setSecret(crudSecret);

    System.out.println("\tsession ID:    " + req.getAuth());
    System.out.println("\tlog parameter: " + req.getLog());
    System.out.println("\toperation:     " + req.getOperation());
    System.out.println("\tresource:      " + req.getResource());
    System.out.println("\tsecret:        " + req.getSecret());
    System.out.println(TGAuthClientCommon.TABLINE);

    try {
      TgCrudCheckAccessResponse response = tgextraCrud.tgCrudCheckAccess(req);

      System.out.println("\tresult: " + response.isResult());

      if (response.isResult()) {
        assertTrue(false);
      }

      System.out.println(TGAuthClientCommon.TABLINE);
      System.out.println(OK);

    } catch (UnknownResourceFault
        | info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault e) {
      printError(e);
      throw e;
    }
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testTgextraCRUDUnregisterResource() {
    assertTrue(NOT_YET_IMPLEMENTED, false);
  }

  //
  // ONLINE-TESTS: Please comment in if needed!
  //

  /**
   * <p>
   * ONLINE TEST: Test getting all the objects from large project.
   * </p>
   * 
   */
  @Test
  @Ignore
  public void testTgextraGetObjectsDigibib() {

    String digibib = "TGPR-372fe6dc-57f2-6cd4-01b5-2c4bbefcfd3c";
    String diginame = "Digitale Bibliothek";

    // String digibib = "TGPR-53fd0966-4bae-45e4-519f-57fcf48e3249";
    // String diginame = "fus tolles projekt";

    System.out.println("\tGetting all objects from project '" + diginame + "': " + digibib);

    GetObjectsRequest req = new GetObjectsRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setProject(digibib);

    try {
      ResourcesetResponse res = tgextra.getObjects(req);
      int size = res.getResource().size();

      System.out.println("Project has got " + size + " entr" + (size != 1 ? "ies" : "y"));
      System.out.println("\tOK");

    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();

      assertTrue(false);
    }
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * <p>
   * Adds a project for testing.
   * </p>
   * 
   * @throws RbacFault
   */
  private static void addProject() throws RbacFault {

    System.out.println("INITIALISING TESTS: CREATING PROJECT");
    System.out.println("\tgetting all projects");

    GetAllProjectsRequest allProjetsRequest = new GetAllProjectsRequest();
    allProjetsRequest.setAuth(rbacSessionId);
    allProjetsRequest.setLog(logParameter);

    long start = System.currentTimeMillis();
    List<ProjectInfo> allProjects = tgextra.getAllProjects(allProjetsRequest).getProject();
    long duration = System.currentTimeMillis() - start;

    System.out.println("\t" + allProjects.size() + " projects existing [" + duration + "ms]");

    System.out.println("\tchecking for junit test project '" + junitProjectName + "'");

    long overallStart = System.currentTimeMillis();

    boolean projectExisting = false;
    for (ProjectInfo pi : allProjects) {
      if (pi.getName().equals(junitProjectName)) {
        projectExisting = true;
        projectId = pi.getId();
      }
    }

    if (projectExisting) {
      System.out.println("\t'" + junitProjectName + "' already exists: " + projectId);
    } else {

      try {
        CreateProjectRequest junitProject1Request = new CreateProjectRequest();
        junitProject1Request.setAuth(rbacSessionId);
        junitProject1Request.setLog(logParameter);
        junitProject1Request.setName(junitProjectName);
        junitProject1Request.setDescription(junitProjectName);

        start = System.currentTimeMillis();
        CreateProjectResponse project1Response = tgextra.createProject(junitProject1Request);
        duration = System.currentTimeMillis() - start;

        projectId = project1Response.getProjectId();

        System.out.println(
            "\tproject " + junitProjectName + " created: " + projectId + " [" + duration + "ms]");

        // AddMemberRequest addMemberRequest = new AddMemberRequest();
        // BooleanResponse addMemberResponse;
        //
        // //
        // // Adding user1 to project1 as Project Manager.
        // //
        // String roleManager =
        // TGAuthClientCommon.ROLE_MANAGER + "," + projectId + ",Projekt-Teilnehmer";
        // addMemberRequest.setRole(roleManager);
        //
        // System.out.println("\tadding member '" + roleManager + "'");
        //
        // start = System.currentTimeMillis();
        // addMemberResponse = tgextra.addMember(addMemberRequest);
        // duration = System.currentTimeMillis() - start;
        // if (!addMemberResponse.isResult()) {
        // System.err.println(ERROR + "adding member failed");
        // assertTrue(false);
        // }
        //
        // System.out.println(OK + " [" + duration + "ms]");
        //
        // // Add user1 to project1 as Editor.
        // String roleEditor =
        // TGAuthClientCommon.ROLE_EDITOR + "," + projectId + ",Projekt-Teilnehmer";
        // addMemberRequest.setAuth(rbacSessionId);
        // addMemberRequest.setLog(logParameter);
        // addMemberRequest.setRole(roleEditor);
        // addMemberRequest.setUsername(username);
        //
        // System.out.println("\tadding member '" + roleEditor + "'");
        //
        // start = System.currentTimeMillis();
        // addMemberResponse = tgextra.addMember(addMemberRequest);
        // duration = System.currentTimeMillis() - start;
        // if (!addMemberResponse.isResult()) {
        // System.err.println(ERROR + "adding member failed");
        // assertTrue(false);
        // }
        //
        // System.out.println(OK + " [" + duration + "ms]");
        //
        // // Add user1 to project1 as Administrator.
        // String roleAdmin = TGAuthClientCommon.ROLE_ADMIN + "," + projectId +
        // ",Projekt-Teilnehmer";
        // addMemberRequest.setRole(roleAdmin);
        //
        // System.out.println("\tadding member '" + roleAdmin + "'");
        //
        // start = System.currentTimeMillis();
        // addMemberResponse = tgextra.addMember(addMemberRequest);
        // duration = System.currentTimeMillis() - start;
        // if (!addMemberResponse.isResult()) {
        // System.err.println(ERROR + "adding member failed");
        // assertTrue(false);
        // }
        //
        // System.out.println(OK + " [" + duration + "ms]");
        //
        // // Add user1 to project1 as Observer.
        // String roleObserver =
        // TGAuthClientCommon.ROLE_OBSERVER + "," + projectId + ",Projekt-Teilnehmer";
        // addMemberRequest.setRole(roleObserver);
        //
        // System.out.println("\tadding member '" + roleObserver + "'");
        //
        // start = System.currentTimeMillis();
        // addMemberResponse = tgextra.addMember(addMemberRequest);
        // duration = System.currentTimeMillis() - start;
        // if (!addMemberResponse.isResult()) {
        // System.err.println(ERROR + "adding member failed");
        // assertTrue(false);
        // }
        //
        // System.out.println(OK + " [" + duration + "ms]");
        //
        // // More output.
        // System.out.println("\tadded " + username + " to project " + junitProjectName
        // + " with roles '" + TGAuthClientCommon.ROLE_EDITOR + "', '"
        // + TGAuthClientCommon.ROLE_ADMIN + "', '" + TGAuthClientCommon.ROLE_MANAGER + "', and '"
        // + TGAuthClientCommon.ROLE_OBSERVER + "'");
        // } catch (RbacFault e) {
        // System.err.println(ERROR + e.getClass().getName() + " --> " + e.getMessage() + " ["
        // + (System.currentTimeMillis() - overallStart) + "ms]");
        // throw e;
      } catch (SOAPFaultException e) {
        System.err.println(ERROR + e.getClass().getName() + " --> " + e.getMessage() + " ["
            + (System.currentTimeMillis() - overallStart) + "ms]");
        throw e;
      } catch (RuntimeException e) {
        System.err.println(ERROR + e.getClass().getName() + " --> " + e.getMessage() + " ["
            + (System.currentTimeMillis() - overallStart) + "ms]");
        throw e;
      }
    }
  }

  /**
   * <p>
   * Test deleting the test project.
   * </p>
   * 
   * @throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault
   * @throws NotEmptyFault
   */
  private static void deleteProject()
      throws info.textgrid.namespaces.middleware.tgauth.AuthenticationFault, NotEmptyFault {

    System.out.println("DELETING PROJECT '" + junitProjectName + "'");

    DeleteProjectRequest req = new DeleteProjectRequest();
    req.setAuth(rbacSessionId);
    req.setLog(logParameter);
    req.setProject(projectId);

    BooleanResponse res = tgextra.deleteProject(req);

    if (!res.isResult()) {
      System.err.println(ERROR + "project could not be deleted");
    } else {
      System.out.println(OK);
    }
  }

  /**
   * <p>
   * Initialises a TG-extra stub with given timeout.
   * </p>
   * 
   * @param theEndpoint
   */
  private static void initTgextraService(URL theEndpoint, int theTimeout) {
    if (tgextra == null) {

      tgextra = TGAuthClientUtilities.getTgextra(theEndpoint, theTimeout);

      if (theTimeout > 0) {
        System.out.println("TGEXTRA stub created with timeout " + theTimeout + "ms");
      } else {
        System.out.println("TGEXTRA stub created");
      }
    } else {
      System.out.println("TGEXTRA stub existing");
    }
  }

  /**
   * <p>
   * Initialises the TG-extra CRUD stub.
   * </p>
   * 
   * @param theEndpoint
   */
  private static void initTgextraCrudService(URL theEndpoint, int theTimeout) {
    if (tgextraCrud == null) {
      tgextraCrud = TGAuthClientCrudUtilities.getTgextraCrud(theEndpoint);

      if (theTimeout > 0) {
        System.out.println("TGEXTRA_CRUD stub created with timeout " + theTimeout + "ms");
      } else {
        System.out.println("TGEXTRA_CRUD stub created");
      }
    } else {
      System.out.println("TGEXTRA_CRUD stub existing");
    }
  }

  /**
   * @param e
   */
  private static void printError(Exception e) {
    System.err.println(ERROR + e.getClass().getName() + " --> " + e.getMessage());
  }

}
